package org.eclipse.jdt.internal.junit.ui;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Iterator;

import junit.extensions.eclipse.diff.JUnitDiffPlugin;

public class TestRunnerViewPartExtension implements ITestIdSelectionListener {
    private TestRunnerViewPart part;
    private ITestSelectionListener listener;

    private TestIdSelectionBroadcaster failureRunViewBroadcaster;
    private TestIdSelectionBroadcaster hierarchyRunViewBroadcaster;
    private TestIdSelectionBroadcaster failureTraceViewBroadcaster;

    public TestRunnerViewPartExtension(TestRunnerViewPart part) {
        this.part = part;
        setUpTestRunViewBroadcasters();
        setUpFailureTraceViewBroadcaster();
    }

    public void dispose() {
        if (failureRunViewBroadcaster != null)
            failureRunViewBroadcaster.dispose();
        if (hierarchyRunViewBroadcaster != null)
            hierarchyRunViewBroadcaster.dispose();
        if (failureTraceViewBroadcaster != null)
            failureTraceViewBroadcaster.dispose();
    }

    public void testIdChanged(String testId) {
        if (listener == null)
            return;
        TestRunInfo info = part.getTestInfo(testId);
        listener.testChanged(info);
    }

    public void setTestSelectionListener(ITestSelectionListener listener) {
        this.listener = listener;
    }

    public String getSelectedTestId() {
        Object testRunView = getField("fActiveRunView");
        if (testRunView == null)
            return null;
        return TestRunViewBroadcaster.getSelectedTestId(testRunView);
    }

    public TestRunInfo getSelectedTestRunInfo() {
        String testId = getSelectedTestId();
        if (testId == null)
            return null;
        return part.getTestInfo(testId);
    }

    private void setUpTestRunViewBroadcasters() {
        Collection views = (Collection) getField("fTestRunViews");
        if (views == null)
            return;
        for (Iterator i = views.iterator(); i.hasNext();) {
            Object o = i.next();
            if (failureRunViewBroadcaster == null
                && o.getClass().equals(FailureRunView.class)) {
                failureRunViewBroadcaster = new TestRunViewBroadcaster(o, FailureRunView.class, "fTable");
                failureRunViewBroadcaster.setTestIdSelectionListener(this);
            } else if (
                hierarchyRunViewBroadcaster == null
                    && o.getClass().equals(HierarchyRunView.class)) {
                hierarchyRunViewBroadcaster = new TestRunViewBroadcaster(o, HierarchyRunView.class, "fTree");
                hierarchyRunViewBroadcaster.setTestIdSelectionListener(this);
            }
        }
    }

    private void setUpFailureTraceViewBroadcaster() {
        Object failureTraceView = getField("fFailureView");
        if (failureTraceView == null)
            return;
        failureTraceViewBroadcaster = 
            new TestIdSelectionBroadcaster(failureTraceView, FailureTraceView.class, "fTable") {
                protected String getSelectedTestId() {
                    return TestRunnerViewPartExtension.this.getSelectedTestId();
                }
            };
        failureTraceViewBroadcaster.setTestIdSelectionListener(this);
    }

    private Object getField(String fieldName) {
        try {
            Field f =
                TestRunnerViewPart.class.getDeclaredField(fieldName);
            f.setAccessible(true);
            return f.get(part);
        } catch (Throwable e) {
            String message = "field " + fieldName + " not found in TestRunnerViewPart.";
            JUnitDiffPlugin.getDefault().logSystemErrorMessage(message, this);
            return null;
        }
    }
}
